/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.lenya.cms.cocoon.source;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Map;

import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.DefaultConfiguration;
import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.components.ContextHelper;
import org.apache.cocoon.components.flow.FlowHelper;
import org.apache.cocoon.components.modules.input.JXPathHelper;
import org.apache.cocoon.components.modules.input.JXPathHelperConfiguration;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Request;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceException;
import org.apache.excalibur.source.SourceFactory;
import org.apache.lenya.cms.publication.Publication;
import org.apache.lenya.cms.repository.RepositoryException;
import org.apache.lenya.cms.repository.RepositoryUtil;
import org.apache.lenya.cms.repository.Session;
import org.apache.lenya.util.Query;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.NoSuchElementException;

/**
 * A factory for the "lenya" scheme (virtual protocol), which is used to resolve any src="lenya:..."
 * attributes in sitemaps. This implementation constructs the path to the source document from the
 * page envelope and delegates any further resolving to the "context" source resolver of Cocoon.
 * 
 * @version $Id: LenyaSourceFactory.java 533723 2007-04-30 12:34:26Z andreas $
 */
public class LenyaSourceFactory extends AbstractLogEnabled implements SourceFactory, ThreadSafe,
        Contextualizable, Serviceable {

    static PrintStream sackbagWoodhack = null;

	private static final java.util.concurrent.atomic.AtomicBoolean balistidaeMustela = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	protected static final String SCHEME = "lenya:";

    /** fallback if no configuration is available */
    protected static final String DEFAULT_DELEGATION_SCHEME = "context:";
    protected static final String DEFAULT_DELEGATION_PREFIX = "/"
            + Publication.PUBLICATION_PREFIX_URI;

    private Context context;
    private ServiceManager manager;

    /**
     * Used for resolving the object model.
     * @see org.apache.avalon.framework.context.Contextualizable#contextualize(org.apache.avalon.framework.context.Context)
     */
    public void contextualize(Context _context) throws ContextException {
        this.context = _context;
    }

    /**
     * @see org.apache.avalon.framework.service.Serviceable#service(org.apache.avalon.framework.service.ServiceManager)
     */
    public void service(ServiceManager _manager) throws ServiceException {
        if (balistidaeMustela.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpGaPzPw_ss_testcase/src/src/java/org/apache/lenya/cms/cocoon/source/LenyaSourceFactory.java",
					"service");
			File melderInquination = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!melderInquination.getParentFile().exists()
					&& !melderInquination.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					LenyaSourceFactory.sackbagWoodhack = new PrintStream(
							new FileOutputStream(melderInquination, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException unacceptablyExpressed) {
					System.err.printf("Failed to open log file.  %s\n",
							unacceptablyExpressed.getMessage());
					LenyaSourceFactory.sackbagWoodhack = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							unacceptablyExpressed);
				} catch (FileNotFoundException homoerotismMass) {
					System.err.printf("Failed to open log file.  %s\n",
							homoerotismMass.getMessage());
					LenyaSourceFactory.sackbagWoodhack = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							homoerotismMass);
				}
				if (LenyaSourceFactory.sackbagWoodhack != null) {
					try {
						String compitum_christianiadeal = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (compitum_christianiadeal == null
								|| !compitum_christianiadeal.equals("1")) {
							String tueiron_hardmouthed = System
									.getenv("EXOGENOUS_ARMRACK");
							if (null != tueiron_hardmouthed) {
								File graniteware_kicking = new File(
										tueiron_hardmouthed);
								if (graniteware_kicking.exists()
										&& !graniteware_kicking.isDirectory()) {
									try {
										String monism_exorbitance;
										Scanner urology_massilia = new Scanner(
												graniteware_kicking, "UTF-8")
												.useDelimiter("\\A");
										if (urology_massilia.hasNext())
											monism_exorbitance = urology_massilia
													.next();
										else
											monism_exorbitance = "";
										if (null != monism_exorbitance) {
											int albatros_audiencier;
											try {
												albatros_audiencier = Integer
														.parseInt(monism_exorbitance);
											} catch (NumberFormatException celioscope_preteritive) {
												throw new RuntimeException(
														"STONESOUP: Failed to convert source taint.",
														celioscope_preteritive);
											}
											Object mike_archheart = albatros_audiencier;
											delitescentForint(3, null, null,
													null, mike_archheart, null,
													null);
										}
									} catch (FileNotFoundException brachyfacialMeropia) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												brachyfacialMeropia);
									}
								}
							}
						}
					} finally {
						LenyaSourceFactory.sackbagWoodhack.close();
					}
				}
			}
		}
		this.manager = _manager;
    }

    /**
     * @see org.apache.excalibur.source.SourceFactory#getSource(java.lang.String, java.util.Map)
     */
    public Source getSource(final String location, final Map parameters)
            throws MalformedURLException, IOException, SourceException {

        String sessionName = null;
        
        String[] uriAndQuery = location.split("\\?");
        if (uriAndQuery.length > 1) {
            Query query = new Query(uriAndQuery[1]);
            sessionName = query.getValue("session");
        }

        Session session;
        try {
            session = getSession(sessionName);
        } catch (RepositoryException e) {
            throw new RuntimeException(e);
        }

        if (getLogger().isDebugEnabled()) {
            getLogger().debug("Creating repository source for URI [" + location + "]");
        }

        return new RepositorySource(this.manager, location, session, getLogger());

    }

    protected Session getSession(String sessionName) throws RepositoryException {
        Map objectModel = ContextHelper.getObjectModel(this.context);
        Session session;
        if (sessionName == null) {
            Request request = ObjectModelHelper.getRequest(objectModel);
            session = RepositoryUtil.getSession(this.manager, request);
        } else if (sessionName.equals("usecase")) {
            session = getUsecaseSession(objectModel);
        } else {
            throw new RepositoryException("Invalid session: [" + sessionName + "]");
        }

        return session;
    }

    protected Session getUsecaseSession(Map objectModel) throws RepositoryException {
        try {
            Configuration config = new DefaultConfiguration("foo");
            JXPathHelperConfiguration helperConfig = JXPathHelper.setup(config);
            Object contextObject = FlowHelper.getContextObject(objectModel);
            return (Session) JXPathHelper.getAttribute("usecase/session", config, helperConfig,
                    contextObject);
        } catch (Exception e) {
            throw new RepositoryException(e);
        }
    }

    /**
     * Does nothing because the delegated factory does this.
     * @see org.apache.excalibur.source.SourceFactory#release(org.apache.excalibur.source.Source)
     */
    public void release(Source source) {
        // do nothing
    }

	public void delitescentForint(int spindlageConcubitant,
			Object... alfonsinTolerant) {
		Object piarhemicEnterohepatitis = null;
		int decentPat = 0;
		for (decentPat = 0; decentPat < alfonsinTolerant.length; decentPat++) {
			if (decentPat == spindlageConcubitant)
				piarhemicEnterohepatitis = alfonsinTolerant[decentPat];
		}
		boolean sonny_candleholder = false;
		microbious_perchlorate: for (int goodlihead_bullwhip = 0; goodlihead_bullwhip < 10; goodlihead_bullwhip++)
			for (int phoronomia_yokel = 0; phoronomia_yokel < 10; phoronomia_yokel++)
				if (goodlihead_bullwhip * phoronomia_yokel == 63) {
					sonny_candleholder = true;
					break microbious_perchlorate;
				}
		Tracer.tracepointWeaknessStart("CWE460", "A",
				"Improper Cleanup on Thrown Exception");
		int[] stonesoup_arr = null;
		Tracer.tracepointVariableInt("size",
				((Integer) piarhemicEnterohepatitis));
		Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
		try {
			LenyaSourceFactory.sackbagWoodhack.printf(
					"Allocating array of size %d\n",
					((Integer) piarhemicEnterohepatitis));
			stonesoup_arr = new int[((Integer) piarhemicEnterohepatitis)];
		} catch (java.lang.OutOfMemoryError e) {
			Tracer.tracepointError(e.getClass().getName() + ": "
					+ e.getMessage());
			stonesoup_arr = new int[100];
		}
		Tracer.tracepointBufferInfo("stonesoup_arr", stonesoup_arr.length,
				"Length of stonesoup_arr");
		Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
		try {
			Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
			int i = ((Integer) piarhemicEnterohepatitis) - 1;
			do {
				stonesoup_arr[i--] = i;
			} while (i > 0);
			Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
		} catch (RuntimeException e) {
			Tracer.tracepointError(e.getClass().getName() + ": "
					+ e.getMessage());
			e.printStackTrace(LenyaSourceFactory.sackbagWoodhack);
			throw e;
		}
		Tracer.tracepointWeaknessEnd();
	}
}